Stan Math Library: stan/math/prim/scal/prob/neg_binomial_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

neg_binomial_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_NEG_BINOMIAL_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_NEG_BINOMIAL_LOG_HPP
3
4 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
5 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
6 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
7 #include <stan/math/prim/scal/err/check_nonnegative.hpp>
8 #include <stan/math/prim/scal/err/check_positive_finite.hpp>
9 #include <stan/math/prim/scal/fun/constants.hpp>
10 #include <stan/math/prim/scal/fun/value_of.hpp>
11 #include <stan/math/prim/scal/fun/binomial_coefficient_log.hpp>
12 #include <stan/math/prim/scal/fun/multiply_log.hpp>
13 #include <stan/math/prim/scal/fun/digamma.hpp>
14 #include <stan/math/prim/scal/fun/lgamma.hpp>
15 #include <stan/math/prim/scal/fun/lbeta.hpp>
16 #include <stan/math/prim/scal/meta/length.hpp>
17 #include <stan/math/prim/scal/meta/VectorView.hpp>
18 #include <stan/math/prim/scal/meta/VectorBuilder.hpp>
19 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
20 #include <stan/math/prim/scal/meta/return_type.hpp>
21 #include <stan/math/prim/scal/meta/include_summand.hpp>
22 #include <stan/math/prim/scal/fun/grad_reg_inc_beta.hpp>
23 #include <stan/math/prim/scal/fun/inc_beta.hpp>
24 #include <stan/math/prim/scal/meta/max_size.hpp>
25 #include <boost/math/special_functions/digamma.hpp>
26 #include <boost/random/negative_binomial_distribution.hpp>
27 #include <boost/random/variate_generator.hpp>
28 #include <cmath>
29
30 namespace stan {
31
32 namespace math {
33
34 // NegBinomial(n|alpha, beta) [alpha > 0; beta > 0; n >= 0]
35 template <bool propto,
36 typename T_n,
37 typename T_shape, typename T_inv_scale>
38 typename return_type<T_shape, T_inv_scale>::type
39 neg_binomial_log(const T_n& n,
40 const T_shape& alpha,
41 const T_inv_scale& beta) {
42 typedef typename stan::partials_return_type<T_n, T_shape,
43 T_inv_scale>::type
44 T_partials_return;
45
46 static const char* function("stan::math::neg_binomial_log");
47
48 using stan::math::check_positive_finite;
49 using stan::math::check_nonnegative;
50 using stan::math::value_of;
51 using stan::math::check_consistent_sizes;
52 using stan::math::include_summand;
53
54 // check if any vectors are zero length
55 if (!(stan::length(n)
56 && stan::length(alpha)
57 && stan::length(beta)))
58 return 0.0;
59
60 T_partials_return logp(0.0);
61 check_nonnegative(function, "Failures variable", n);
62 check_positive_finite(function, "Shape parameter", alpha);
63 check_positive_finite(function, "Inverse scale parameter", beta);
64 check_consistent_sizes(function,
65 "Failures variable", n,
66 "Shape parameter", alpha,
67 "Inverse scale parameter", beta);
68
69 // check if no variables are involved and prop-to
70 if (!include_summand<propto, T_shape, T_inv_scale>::value)
71 return 0.0;
72
73 using stan::math::multiply_log;
74 using stan::math::binomial_coefficient_log;
75 using stan::math::digamma;
76 using stan::math::lgamma;
77 using std::log;
78 using std::log;
79
80 // set up template expressions wrapping scalars into vector views
81 VectorView<const T_n> n_vec(n);
82 VectorView<const T_shape> alpha_vec(alpha);
83 VectorView<const T_inv_scale> beta_vec(beta);
84 size_t size = max_size(n, alpha, beta);
85
86 OperandsAndPartials<T_shape, T_inv_scale>
87 operands_and_partials(alpha, beta);
88
89 size_t len_ab = max_size(alpha, beta);
90 VectorBuilder<true, T_partials_return, T_shape, T_inv_scale>
91 lambda(len_ab);
92 for (size_t i = 0; i < len_ab; ++i)
93 lambda[i] = value_of(alpha_vec[i]) / value_of(beta_vec[i]);
94
95 VectorBuilder<true, T_partials_return, T_inv_scale>
96 log1p_beta(length(beta));
97 for (size_t i = 0; i < length(beta); ++i)
98 log1p_beta[i] = log1p(value_of(beta_vec[i]));
99
100 VectorBuilder<true, T_partials_return, T_inv_scale>
101 log_beta_m_log1p_beta(length(beta));
102 for (size_t i = 0; i < length(beta); ++i)
103 log_beta_m_log1p_beta[i] = log(value_of(beta_vec[i])) - log1p_beta[i];
104
105 VectorBuilder<true, T_partials_return, T_inv_scale, T_shape>
106 alpha_times_log_beta_over_1p_beta(len_ab);
107 for (size_t i = 0; i < len_ab; ++i)
108 alpha_times_log_beta_over_1p_beta[i]
109 = value_of(alpha_vec[i])
110 * log(value_of(beta_vec[i])
111 / (1.0 + value_of(beta_vec[i])));
112
113 VectorBuilder<!is_constant_struct<T_shape>::value,
114 T_partials_return, T_shape>
115 digamma_alpha(length(alpha));
116 if (!is_constant_struct<T_shape>::value) {
117 for (size_t i = 0; i < length(alpha); ++i)
118 digamma_alpha[i] = digamma(value_of(alpha_vec[i]));
119 }
120
121 VectorBuilder<!is_constant_struct<T_shape>::value,
122 T_partials_return, T_inv_scale> log_beta(length(beta));
123 if (!is_constant_struct<T_shape>::value) {
124 for (size_t i = 0; i < length(beta); ++i)
125 log_beta[i] = log(value_of(beta_vec[i]));
126 }
127
128 VectorBuilder<!is_constant_struct<T_inv_scale>::value,
129 T_partials_return, T_shape, T_inv_scale>
130 lambda_m_alpha_over_1p_beta(len_ab);
131 if (!is_constant_struct<T_inv_scale>::value) {
132 for (size_t i = 0; i < len_ab; ++i)
133 lambda_m_alpha_over_1p_beta[i] =
134 lambda[i]
135 - (value_of(alpha_vec[i])
136 / (1.0 + value_of(beta_vec[i])));
137 }
138
139 for (size_t i = 0; i < size; i++) {
140 if (alpha_vec[i] > 1e10) { // reduces numerically to Poisson
141 if (include_summand<propto>::value)
142 logp -= lgamma(n_vec[i] + 1.0);
143 if (include_summand<propto, T_shape, T_inv_scale>::value)
144 logp += multiply_log(n_vec[i], lambda[i]) - lambda[i];
145
146 if (!is_constant_struct<T_shape>::value)
147 operands_and_partials.d_x1[i]
148 += n_vec[i] / value_of(alpha_vec[i])
149 - 1.0 / value_of(beta_vec[i]);
150 if (!is_constant_struct<T_inv_scale>::value)
151 operands_and_partials.d_x2[i]
152 += (lambda[i] - n_vec[i]) / value_of(beta_vec[i]);
153 } else { // standard density definition
154 if (include_summand<propto, T_shape>::value)
155 if (n_vec[i] != 0)
156 logp += binomial_coefficient_log(n_vec[i]
157 + value_of(alpha_vec[i])
158 - 1.0,
159 n_vec[i]);
160 if (include_summand<propto, T_shape, T_inv_scale>::value)
161 logp +=
162 alpha_times_log_beta_over_1p_beta[i]
163 - n_vec[i] * log1p_beta[i];
164
165 if (!is_constant_struct<T_shape>::value)
166 operands_and_partials.d_x1[i]
167 += digamma(value_of(alpha_vec[i]) + n_vec[i])
168 - digamma_alpha[i]
169 + log_beta_m_log1p_beta[i];
170 if (!is_constant_struct<T_inv_scale>::value)
171 operands_and_partials.d_x2[i]
172 += lambda_m_alpha_over_1p_beta[i]
173 - n_vec[i] / (value_of(beta_vec[i]) + 1.0);
174 }
175 }
176 return operands_and_partials.value(logp);
177 }
178
179 template <typename T_n,
180 typename T_shape, typename T_inv_scale>
181 inline
182 typename return_type<T_shape, T_inv_scale>::type
183 neg_binomial_log(const T_n& n,
184 const T_shape& alpha,
185 const T_inv_scale& beta) {
186 return neg_binomial_log<false>(n, alpha, beta);
187 }
188 }
189 }
190 #endif
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
lgamma.hpp
stan::math::binomial_coefficient_log
fvar< T > binomial_coefficient_log(const fvar< T > &x1, const fvar< T > &x2)
Definition: binomial_coefficient_log.hpp:16
stan::math::lgamma
fvar< T > lgamma(const fvar< T > &x)
Definition: lgamma.hpp:15
check_positive_finite.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
digamma.hpp
stan::math::log
fvar< T > log(const fvar< T > &x)
Definition: log.hpp:15
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
binomial_coefficient_log.hpp
is_constant_struct.hpp
return_type.hpp
stan::math::include_summand
Template metaprogram to calculate whether a summand needs to be included in a proportional (log) prob...
Definition: include_summand.hpp:36
stan::return_type::type
boost::math::tools::promote_args< typename scalar_type< T1 >::type, typename scalar_type< T2 >::type, typename scalar_type< T3 >::type, typename scalar_type< T4 >::type, typename scalar_type< T5 >::type, typename scalar_type< T6 >::type >::type type
Definition: return_type.hpp:27
include_summand.hpp
multiply_log.hpp
check_nonnegative.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::partials_return_type
Definition: partials_return_type.hpp:16
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
VectorView.hpp
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
VectorBuilder.hpp
partials_return_type.hpp
OperandsAndPartials.hpp
length.hpp
grad_reg_inc_beta.hpp
stan::math::multiply_log
fvar< T > multiply_log(const fvar< T > &x1, const fvar< T > &x2)
Definition: multiply_log.hpp:15
stan::VectorBuilder
VectorBuilder allocates type T1 values to be used as intermediate values.
Definition: VectorBuilder.hpp:28
constants.hpp
stan::math::size
int size(const std::vector< T > &x)
Return the size of the specified standard vector.
Definition: size.hpp:17
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
lbeta.hpp
stan::math::log1p
fvar< T > log1p(const fvar< T > &x)
Definition: log1p.hpp:16
inc_beta.hpp
stan::math::check_nonnegative
bool check_nonnegative(const char *function, const char *name, const T_y &y)
Return true if y is non-negative.
Definition: check_nonnegative.hpp:66
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
stan::math::neg_binomial_log
return_type< T_shape, T_inv_scale >::type neg_binomial_log(const T_n &n, const T_shape &alpha, const T_inv_scale &beta)
Definition: neg_binomial_log.hpp:39
check_consistent_sizes.hpp
stan::math::check_positive_finite
bool check_positive_finite(const char *function, const char *name, const T_y &y)
Return true if y is positive and finite.
Definition: check_positive_finite.hpp:28
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39
max_size.hpp
stan::math::digamma
fvar< T > digamma(const fvar< T > &x)
Definition: digamma.hpp:16

[ Stan Home Page ] © 2011–2016, Stan Development Team.